# frozen_string_literal: true

require "dwarves/parser/basic_rules"
require "dwarves/parser/4"

module Dwarves
  # Base parser, delegates the actual parsing to versioned classes.
  module Parser
    # A parser error
    class Error < StandardError; end

    # Get a versioned parser.
    #
    # @example
    #   v4 = Dwarves::Parser[4].new
    #
    # @param version [Integer] which version of DWARF to use
    # @return [Subclass of Dwarves::Parser] the versioned subclass
    #
    def self.[] version
      raise Error, "Unsupported DWARF version!" unless version == 4

      V4
    end
  end
end
