# frozen_string_literal: true

require "parslet"

require "dwarves/parser/4/ast"

module Dwarves
  module Parser
    # Transformer to turn the parser output into meaningful classes
    #
    class V4Transformer < Parslet::Transform
      rule(
        compilation_unit_offset: simple(:c),
        length: simple(:l),
        architecture: simple(:arch),
        version: simple(:v),
        abbrev_offset: simple(:ab),
        pointer_size: simple(:p),
        dies: subtree(:d)
      ) do
        Dwarves::Parser::CU.new \
          compilation_unit_offset: c,
          length: l,
          architecture: arch,
          version: v,
          abbrev_offset: ab,
          pointer_size: p,
          dies: d
      end

      rule(
        depth: simple(:d),
        address: simple(:a),
        abbrev_number: simple(:an),
        attributes: subtree(:attrs)
      ) do
        Dwarves::Parser::DIE.new \
          attributes: attrs,
          address: a,
          abbrev_number: an,
          depth: d
      end

      rule(
        depth: simple(:d),
        address: simple(:a),
        abbrev_number: simple(:an),
        type: simple(:t),
        attributes: subtree(:attrs)
      ) do
        Dwarves::Parser::DIE.new \
          tag: t,
          children: [],
          siblings: [],
          attributes: attrs,
          address: a,
          abbrev_number: an,
          depth: d
      end

      rule address: simple(:a), name: simple(:n), description: simple(:d) do
        Dwarves::Parser::Attribute.new name: n, value: d, address: a
      end
    end
  end
end
