# frozen_string_literal: true

require "parslet"

module Dwarves
  module Parser
    # Shared rules common to many parsers.
    #
    # These can be directly included in your parser class, like so
    #
    # @example
    #   class SomeParser < Parslet::Parser
    #     include Dwarves::Parser::BasicRules
    #
    #     # ...
    #   end
    #
    module BasicRules
      include Parslet

      # @!method colon
      rule(:colon) { str ":" }

      # @!method space
      rule(:space) { match('\s').repeat 1 }

      # @!method space?
      rule(:space?) { space.maybe }

      # @!method newline
      rule(:newline) { match('\n').repeat 1 }

      # @!method newlines
      rule(:newlines) { newline.repeat }

      # @!method separator
      rule(:separator) { space? | space }

      alias _ separator

      # @!method digit
      rule(:digit) { match("[0-9]").repeat 1 }

      # @!method integer
      rule(:integer) { digit.repeat }

      # @!method word
      rule(:word) { match('[a-zA-Z\._0-9]').repeat 1 }

      # @!method words
      rule(:words) { word >> (separator >> word).repeat }

      # @!method lparen
      rule(:lparen) { str "(" }

      # @!method rparen
      rule(:rparen) { str ")" }

      # @!method semicolon
      rule(:semicolon) { str ";" }

      # @!method hexadecimal
      rule :hexadecimal do
        str("0x") >> (match("[a-f]") | match("[0-9]")).repeat
      end

      # @!method filename
      rule :filename do
        match('[^\s]').repeat
      end
    end
  end
end
